[llvm-branch-commits] [llvm] f9c3954 - Fix for Bug 48055.

Hiroshi Yamauchi via llvm-branch-commits llvm-branch-commits at lists.llvm.org
Fri Dec 4 11:10:26 PST 2020


Author: Hiroshi Yamauchi
Date: 2020-12-04T11:05:01-08:00
New Revision: f9c3954a6ec5d1066a67aadab848f02b9b78b056

URL: https://github.com/llvm/llvm-project/commit/f9c3954a6ec5d1066a67aadab848f02b9b78b056
DIFF: https://github.com/llvm/llvm-project/commit/f9c3954a6ec5d1066a67aadab848f02b9b78b056.diff

LOG: Fix for Bug 48055.

Differential Revision: https://reviews.llvm.org/D92599

Added: 
    llvm/test/Transforms/GlobalOpt/evaluate-bitcast-4.ll

Modified: 
    llvm/lib/Transforms/Utils/Evaluator.cpp

Removed: 
    


################################################################################
diff  --git a/llvm/test/Transforms/GlobalOpt/evaluate-bitcast-4.ll b/llvm/test/Transforms/GlobalOpt/evaluate-bitcast-4.ll
new file mode 100644
index 000000000000..1e7c89894d2a
--- /dev/null
+++ b/llvm/test/Transforms/GlobalOpt/evaluate-bitcast-4.ll
@@ -0,0 +1,29 @@
+; PR48055. Check that this does not crash.
+; RUN: opt -globalopt %s -disable-output
+target datalayout = "e-m:e-p270:32:32-p271:32:32-p272:64:64-i64:64-f80:128-n8:16:32:64-S128"
+target triple = "x86_64-unknown-linux-gnu"
+
+%struct.g = type opaque
+%struct.a = type { i32 (...)** }
+
+ at l = dso_local global i32 0, align 4
+ at h = external dso_local global %struct.g, align 1
+ at llvm.global_ctors = appending global [1 x { i32, void ()*, i8* }] [{ i32, void ()*, i8* } { i32 65535, void ()* @_GLOBAL__sub_I_bug48055.cc, i8* null }]
+
+; Function Attrs: uwtable
+define internal void @__cxx_global_var_init() {
+entry:
+  %vtable = load i32 (%struct.a*)**, i32 (%struct.a*)*** bitcast (%struct.g* @h to i32 (%struct.a*)***), align 1
+  %0 = load i32 (%struct.a*)*, i32 (%struct.a*)** %vtable, align 8
+  %call = call i32 %0(%struct.a* nonnull dereferenceable(8) bitcast (%struct.g* @h to %struct.a*))
+  store i32 %call, i32* @l, align 4
+  ret void
+}
+
+; Function Attrs: uwtable
+define internal void @_GLOBAL__sub_I_bug48055.cc() {
+entry:
+  call void @__cxx_global_var_init()
+  ret void
+}
+


        


More information about the llvm-branch-commits mailing list
diff  --git a/llvm/lib/Transforms/Utils/Evaluator.cpp b/llvm/lib/Transforms/Utils/Evaluator.cpp
index 6fe29381c71a..a01dc7a4e609 100644
--- a/llvm/lib/Transforms/Utils/Evaluator.cpp
+++ b/llvm/lib/Transforms/Utils/Evaluator.cpp
@@ -183,11 +183,11 @@ evaluateBitcastFromPtr(Constant *Ptr, const DataLayout &DL,
                        std::function<Constant *(Constant *)> Func) {
   Constant *Val;
   while (!(Val = Func(Ptr))) {
     // If Ty is a struct, we can convert the pointer to the struct
     // into a pointer to its first member.
     // FIXME: This could be extended to support arrays as well.
     Type *Ty = cast<PointerType>(Ptr->getType())->getElementType();
-    if (!isa<StructType>(Ty))
+    if (!isa<StructType>(Ty) || cast<StructType>(Ty)->isOpaque())
       break;
 
     IntegerType *IdxTy = IntegerType::get(Ty->getContext(), 32);
@@ -183,11 +183,11 @@ evaluateBitcastFromPtr(Constant *Ptr, const DataLayout &DL,
   Constant *Val;
   while (!(Val = Func(Ptr))) {
-    // If Ty is a struct, we can convert the pointer to the struct
+    // If Ty is a non-opaque struct, we can convert the pointer to the struct
     // into a pointer to its first member.
     // FIXME: This could be extended to support arrays as well.
